
# q-expansion of newform 931.2.a.g, downloaded from the LMFDB on 10 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 931
weight = 2
poly_data = [-3, -1, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0, -1], [2, -1], [3, 0], [0, 0], [-3, 1], [1, 2], [3, 1], [-1, 0], [-3, 0], [3, 3], [1, -1], [-1, 2], [-3, 1], [-10, 0], [3, -4], [0, -3], [-3, 4], [-5, 4], [5, -3], [3, 4], [10, -5], [2, 4], [6, 3], [6, 2], [-5, -2], [9, -2], [7, 4], [-15, 4], [-1, -4], [12, -1], [2, -6], [9, 5], [-6, 12], [-5, 0], [3, -4], [-1, 3], [7, -5], [8, -1], [-9, 2], [6, -6], [9, -9], [13, -1], [0, 11], [-7, -7], [0, -3], [-5, 8], [-10, 1], [1, -6], [9, 7], [10, -4], [-9, 3], [-9, -8], [16, 2], [9, 3], [-6, 3], [-12, 1], [0, -9], [-11, -3], [-4, 2], [3, 0], [-2, -11], [-15, 12], [-17, 1], [-9, 5], [4, 10], [18, -4], [14, -1], [-13, 9], [6, -15], [4, -5], [-3, -3], [9, -15], [-23, 10], [8, 7], [-7, 6], [21, -12], [15, -17], [-14, -10], [-15, 5], [-14, -3], [-15, 0], [-25, -2], [6, -4], [19, -8], [-32, 4], [0, -7], [-12, 5], [20, 1], [6, -3], [5, -18], [18, -15], [3, -1], [23, -8], [-12, 0], [-10, 13], [12, -8], [-6, -12], [-9, -12], [13, -22], [26, -4], [-22, 5], [15, -7], [15, 4], [21, -6], [-7, 2], [-8, -5], [18, -8], [-9, 6], [6, -3], [16, 5], [13, 0], [20, -9], [-27, 7], [16, 11], [-19, 14], [3, 3], [-35, 2], [3, -8], [-3, -6], [-9, 11], [10, -4], [8, 3], [12, -13], [3, 4], [28, 6], [12, 4], [-7, -2], [0, 12], [-23, 22], [-32, 0], [-37, 8], [15, -12], [-22, 21], [2, 0], [15, 4], [16, 4], [36, -6], [4, 12], [-33, 3], [-3, -10], [-14, 20], [-24, -12], [-16, 14], [-45, 0], [-8, -20], [12, -4], [22, -11], [-39, 3], [25, 1], [18, -15], [-22, 6], [-24, 19], [-37, -4], [-33, 6], [-19, 6], [-6, 0], [-31, 10], [0, -5], [-5, -7], [-6, 0], [30, 3], [-21, 11], [29, -1], [-9, 12], [24, -18], [3, 20], [-40, 10], [-26, 9]]
